/****************************************************************************

    game.c
    gameplay stuff

    DoubleDash game
      
    Copyright (C) 2003  Michal Benes
  
    DoubleDash is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.
  
    This code is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
  
    You should have received a copy of the GNU Lesser General Public
    License along with this code; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
                                                                               
*****************************************************************************/

#include "doubledash.h"
#include "display.h"
#include "game.h"
#include "player.h"
#include "sdl.h"
#include "utils.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#define LEVEL_MAX_W 256
#define LEVEL_MAX_H 256

int game_players_num=2;
int game_level=1;
int game_diamonds=1;
int game_level_map[LEVEL_MAX_W][LEVEL_MAX_H];
int game_level_map_state[LEVEL_MAX_W][LEVEL_MAX_H];
int game_level_map_check[LEVEL_MAX_W][LEVEL_MAX_H];
int game_level_w;
int game_level_h;
int game_frame=0;



/*
  Get map dimensions
*/
void game_get_map_dimen( int *mapw, int *maph )
{
  *mapw=game_level_w;
  *maph=game_level_h;
}



/*
  Get map title
*/
int game_get_map_title( int x, int y )
{
  if( x<0 || y<0 || x>=game_level_w || y>=game_level_h ) return MAP_BARRIER;
  return game_level_map[x][y];
}



/*
  Load level map
 */
void game_level_read_map( FILE *level )
{
  int i, j;
  int entry_num=0;
  int exit_num=0;
  char buf[1024];
  char *tbuf;

  game_level_h=0;

  while( !feof(level) && game_level_h<LEVEL_MAX_H )
    {
      if( fgets( buf, 1000, level )==NULL ) break;
      buf[1000]=0;
      tbuf=utils_trim( buf );
      if( tbuf[0]==0 ) break;
      if( game_level_h==0 )
	{
	  game_level_w=strlen(tbuf);
	  if( game_level_w>LEVEL_MAX_W )
	    {
	      fprintf( stderr, "DoubleDash: Level %d too wide\n", game_level );
	      exit(1);	      
	    }
	}

      for( i=0, j=0; i<game_level_w; ++i )
	{
	  switch(tbuf[j])
	    {
	    case '\0':
	    case ' ':
	      game_level_map[i][game_level_h]=MAP_FREE;
	      break;
	    case '.':
	      game_level_map[i][game_level_h]=MAP_DIRT;
	      break;
	    case '#':
	      game_level_map[i][game_level_h]=MAP_BARRIER;
	      break;
	    case '=':
	      game_level_map[i][game_level_h]=MAP_WALL;
	      break;
	    case '*':
	      game_level_map[i][game_level_h]=MAP_DIAMOND;
	      break;
	    case 'O':
	      game_level_map[i][game_level_h]=MAP_BOULDER;
	      break;
	    case 'E':
	      game_level_map[i][game_level_h]=MAP_ENTRY;
	      entry_num++;
	      break;
	    case 'X':
	      game_level_map[i][game_level_h]=MAP_EXIT;
	      exit_num++;
	      break;
	    case 'B':
	      game_level_map[i][game_level_h]=MAP_BUTTERFLY;
	      game_level_map_state[i][game_level_h]=3;
	      break;
	    case 'F':
	      game_level_map[i][game_level_h]=MAP_FIREFLY;
	      game_level_map_state[i][game_level_h]=1;
	      break;
	    }
	  if( tbuf[j] ) ++j;
	}
      
      ++game_level_h;
    }

  if( game_level_h>LEVEL_MAX_H )
    {
      fprintf( stderr, "DoubleDash: Level %d too tall\n", game_level );
      exit(1);	      
    }

  if( entry_num!=4 )
    {
      fprintf( stderr, "DoubleDash: Number of map enters must be four.\n" );
      exit(1);	      
    }
  
  if( exit_num!=1 )
    {
      fprintf( stderr, "DoubleDash: Number of map exits must be one.\n" );
      exit(1);	      
    }

  for( i=0; i<game_level_w; ++i )
    {
      if( game_level_map[i][0]!=MAP_EXIT )
	game_level_map[i][0]=MAP_BARRIER;
      if( game_level_map[i][game_level_h-1]!=MAP_EXIT )
	game_level_map[i][game_level_h-1]=MAP_BARRIER;
    }
  
  for( i=0; i<game_level_h; ++i )
    {
      if( game_level_map[0][i]!=MAP_EXIT )
	game_level_map[0][i]=MAP_BARRIER;
      if( game_level_map[game_level_w-1][i]!=MAP_EXIT )
	game_level_map[game_level_w-1][i]=MAP_BARRIER;
    }
  
}



/*
  Load currentlevel+dif
 */
void game_level_load_relative( int dif )
{
  FILE *level;
  char name[20];
  char buf[1024];
  char *tbuf;
  int i, n, k, l;
  game_diamonds=-1;
  game_level_w=0;
  game_level_h=0;
  
  game_level+=dif;
  sprintf( name, "level%d.lev", game_level );
  level=fopen( utils_fullname(name), "rb" );

  if( level==NULL )
    {
      game_level=1;
      sprintf( name, "level%d.lev", game_level );
      level=fopen( utils_fullname(name), "rb" );
    }

  if( level==NULL )
    {
      fprintf( stderr, "DoubleDash: No level file found\n" );
      exit(1);
    }

  while( !feof(level) )
    {
      fgets( buf, 1000, level );
      buf[1000]=0;
      tbuf=utils_trim( buf );
      if( tbuf[0]!='\0' )
	{
	  if( utils_get_param( tbuf, "level", &n ) )
	    {
	      game_level_read_map(level);
	    }
	  else
	    {
	      if( utils_get_param( tbuf, "diamonds", &n ) )
		{
		  game_diamonds=(n+game_players_num-1)/game_players_num;
                  if( game_players_num==2 ) game_diamonds=(n+2)/3;
		}
	      else
		{
		  fprintf( stderr, "DoubleDash: Can not parse level line:\n%s\n", tbuf );
		  exit(1);
		}
	    }
	}
    }

  fclose(level);

  if( game_level_w<=0 || game_level_h<=0 )
    {
      fprintf( stderr, "DoubleDash: Level %d map not found in level file.\n", game_level );
      exit(1);
    }
    
  if( game_diamonds<=0 )
    {
      fprintf( stderr, "DoubleDash: Number of diamonds to collect not specified for level %d.\n", game_level );
      exit(1);
    }

  /* init players */
  for( i=0; i<4; ++i )
    player_dead(i);
  

  for( i=0; i<game_players_num; ++i )
    {
      n=utils_rnd(4-i);
      for( l=0; l<game_level_h; ++l )
	for( k=0; k<game_level_w; ++k )
	  if( game_level_map[k][l]==MAP_ENTRY )
	    {
	      if( n==0 )
		{
		  game_level_map[k][l]=MAP_EGG;
		  player_init( i, k, l, game_level_w/2, game_level_h/2 );
		  goto donextplayer;
		}
	      else
		{
		  --n;
		}
	    }
    donextplayer:
	
      /*
	do nothing
       */

    }

  for( l=0; l<game_level_h; ++l )
    for( k=0; k<game_level_w; ++k )
      if( game_level_map[k][l]==MAP_ENTRY )
	game_level_map[k][l]=MAP_BARRIER;


  sdl_reset_keyboard();

  game_frame=0;

}


/*
  check if boulder or diamond can fall here
 */
int game_check_fall( int i, int j )
{
  if( game_level_map[i][j+1]!=MAP_DIAMOND &&
      game_level_map[i][j+1]!=MAP_BOULDER &&
      game_level_map[i][j+1]!=MAP_WALL &&
      game_level_map[i][j+1]!=MAP_BARRIER )
    return 0;

  if( game_level_map[i-1][j]==MAP_FREE && game_level_map[i-1][j+1]==MAP_FREE )
    return -1;

  if( game_level_map[i+1][j]==MAP_FREE && game_level_map[i+1][j+1]==MAP_FREE )
    return 1;

  return 0;
}



/*
  Load next level
 */
void game_level_load_next( void )
{
  game_level_load_relative(1);
}


/*
  Reload current level
 */
void game_level_reload( void )
{
  game_level_load_relative(0);
}




/*
  Make explosion
 */
void game_explosion( int x, int y, int type )
{
  int i, j;

  for( j=-1; j<=1; ++j )
    for( i=-1; i<=1; ++i )
      if( game_level_map[x+i][y+j]!=MAP_BARRIER &&
	  game_level_map[x+i][y+j]!=MAP_EGG &&
	  game_level_map[x+i][y+j]!=MAP_EXIT )
	{
	  if( game_level_map[x+i][y+j]==MAP_PLAYER )
	    player_dead( player_get_id( x+i, y+j ) );
	  game_level_map[x+i][y+j]=type;
	  game_level_map_check[x+i][y+j]=1;
	}
}


/*
  Move fly object
 */
void game_fly_move( int i, int j, int object )
{
  int dir;
  int k;
  int d;
  int dx=0, dy=0;

  dir=game_level_map_state[i][j];
  if( object==MAP_FIREFLY )
    d=-1;
  else 
    d=1;
  
  dir=(dir+4+d)%4;

  for( k=0; k<4; ++k, dir=(dir+4-d)%4 )
    {
      switch( dir )
	{
	case 0:
	  dx=0;
	  dy=-1;
	  break;
	case 1:
	  dx=1;
	  dy=0;
	  break;
	case 2:
	  dx=0;
	  dy=1;
	  break;
	case 3:
	  dx=-1;
	  dy=0;
	  break;
	default:
	  break;
	}
      
      if( game_level_map[i+dx][j+dy]==MAP_FREE )
	{
	  game_level_map[i][j]=MAP_FREE;
	  game_level_map[i+dx][j+dy]=object;
	  game_level_map_check[i+dx][j+dy]=1;
	  game_level_map_state[i+dx][j+dy]=dir;
	  break;
	}
    }

}


/*
  Plays level
 */
void game_play_level(void)
{
  int last_ticks;
  int ticks;
  int i, j;
  int players;
  int object;
  int tmpobject;
  int dx, dy, dig;
  int id;


 play_level:

  last_ticks=SDL_GetTicks();
  while( 1 )
    {      
      sdl_poll_events();

      for( j=0; j<game_level_h; ++j )
	for( i=0; i<game_level_w; ++i )
	  game_level_map_check[i][j]=0;

      players=0;


      for( j=0; j<game_level_h; ++j )
	for( i=0; i<game_level_w; ++i )
	  if( !game_level_map_check[i][j] )
	    {
	      game_level_map_check[i][j]=1;
	      object=game_level_map[i][j];
	      switch( object )
		{
		case MAP_BOULDER:
		case MAP_FALLING_BOULDER:
		case MAP_DIAMOND:
		case MAP_FALLING_DIAMOND:
		  if( object==MAP_FALLING_DIAMOND ||
		      object==MAP_FALLING_BOULDER )
		    {
		      tmpobject=game_level_map[i][j+1];
		      if( tmpobject==MAP_PLAYER ||
			  tmpobject==MAP_BUTTERFLY )
			{
			  game_explosion( i, j+1, MAP_DIAMOND_EXPLOSION );
			  break;
			}
		      if( tmpobject==MAP_FIREFLY )
			{
			  game_explosion( i, j+1, MAP_FREE_EXPLOSION );
			  break;
			}
		    }
		  if( object==MAP_DIAMOND )
		    object=MAP_FALLING_DIAMOND;
		  if( object==MAP_BOULDER )
		    object=MAP_FALLING_BOULDER;
		  if( game_level_map[i][j+1]==MAP_FREE )
		    {
		      game_level_map[i][j]=MAP_FREE;
		      game_level_map[i][j+1]=object;
		      game_level_map_check[i][j+1]=1;
		      break;		      
		    }

		  if( (dx=game_check_fall( i, j ))!=0 )
		    {
		      game_level_map[i][j]=MAP_FREE;
		      game_level_map[i+dx][j]=object;
		      game_level_map_check[i+dx][j]=1;
		      break;
		    }

		  /* object is not falling */
		  if( object==MAP_FALLING_DIAMOND )
		    object=MAP_DIAMOND;
		  if( object==MAP_FALLING_BOULDER )
		    object=MAP_BOULDER;
		  game_level_map[i][j]=object;
		  break;

		case MAP_EGG:
		  if( game_frame>=BORN_TIME ) game_level_map[i][j]=MAP_PLAYER;
                  players++;		
		  break;

		case MAP_PLAYER:
		  id=player_get_id( i, j );
		  players++;
		  player_get_dir( id, &dx, &dy, &dig );
		  if( dx!=0 || dy!=0 )
		    {
		      tmpobject=game_level_map[i+dx][j+dy];
		      if( tmpobject==MAP_FREE || tmpobject==MAP_DIRT ||
			  tmpobject==MAP_DIAMOND || 
			  ( dy==0 && !dig && tmpobject==MAP_BOULDER && game_level_map[i+2*dx][j]==MAP_FREE && !utils_rnd(3) ) ||
			  ( tmpobject==MAP_EXIT && !dig && player_get_success(id) ) )
			{
			  if( tmpobject==MAP_EXIT )
			    {
			      player_add_total_score( id, player_get_score(id)*100/game_diamonds );
			      game_level_load_next();
			      goto play_level;
			    }
			  if( tmpobject==MAP_DIAMOND )
			    {
			      player_got_diamond(id);
			      if( player_get_score(id)==game_diamonds )
				player_set_success(id);
			    }
			  if( tmpobject==MAP_BOULDER )
			    {
			      game_level_map[i+2*dx][j]=MAP_BOULDER;
			      game_level_map_check[i+2*dx][j]=1;
			    }
			  player_can_move( id, 1 );
			  game_level_map_check[i+dx][j+dy]=1;
			  if( dig )
			    {
			      game_level_map[i+dx][j+dy]=MAP_FREE;
			    }
			  else
			    {
			      game_level_map[i][j]=MAP_FREE;
			      game_level_map[i+dx][j+dy]=MAP_PLAYER;
			      player_move( id );
			    }
			}
		      else
			{
			  if( tmpobject==MAP_BUTTERFLY )
			    game_explosion( i+dx, j+dy, MAP_DIAMOND_EXPLOSION );
			  if( tmpobject==MAP_FIREFLY )
			    game_explosion( i+dx, j+dy, MAP_FREE_EXPLOSION );
			  player_can_move( id, 0 );
			}
		    }
		  break;

		case MAP_DIAMOND_EXPLOSION:
		  game_level_map[i][j]=MAP_DIAMOND;
		  break;

		case MAP_FREE_EXPLOSION:
		  game_level_map[i][j]=MAP_FREE;
		  break;
		  
		case MAP_BUTTERFLY:
		case MAP_FIREFLY:
		  if( game_level_map[i-1][j]==MAP_PLAYER ||
		      game_level_map[i+1][j]==MAP_PLAYER ||
		      game_level_map[i][j-1]==MAP_PLAYER ||
		      game_level_map[i][j+1]==MAP_PLAYER ||
		      game_level_map[i][j-1]==MAP_FALLING_BOULDER ||
		      game_level_map[i][j-1]==MAP_FALLING_DIAMOND )
		    {
		      if( object==MAP_BUTTERFLY )
			game_explosion( i, j, MAP_DIAMOND_EXPLOSION );
		      else
			game_explosion( i, j, MAP_FREE_EXPLOSION );
		      break;
		    }
		  game_fly_move( i, j, object );
		  break;

		default:
		  /* do nothing */
		  break;
		}
	    }

      if( players==0 || (players==1 && game_players_num>1) )
	{
	  for( j=0; j<game_level_h; ++j )
	    for( i=0; i<game_level_w; ++i )
	      if( game_level_map[i][j]==MAP_PLAYER )
		{
		  id=player_get_id( i, j );
		  player_add_total_score( id, player_get_score(id)*150/game_diamonds );
		}
	  game_level_reload();
	  goto play_level;
	}

      for( i=0; i<2; ++i )
	{
	  for( id=0; id<game_players_num; ++id )
	    player_next_frame(id,i==0);

	  display_update( game_players_num, game_frame );
	  ticks=SDL_GetTicks();
	  if( last_ticks+100-ticks>0 )
	    SDL_Delay(last_ticks+100-ticks);
	  last_ticks=ticks;
	  game_frame++;
	}

      if( sdl_esc_pressed() )
	{
	  if( game_frame<22 ) break;
	  game_level_reload();
	  goto play_level;
	}

    }
}


/*
  Start a new game
 */

void game_start(void)
{
  Sdl_Sprite *background;
  game_level=0;
  player_null_score();

  background=sdl_sprite_load( "game_background.png", 1, 1 );
  sdl_sprite_draw( background, 0, 0, 0 );
  sdl_flip();
  sdl_sprite_draw( background, 0, 0, 0 );
  sdl_sprite_free( background );

  game_level_load_next();

  game_play_level();

}



/*
  Set the number of players
 */
void game_set_players_num( int n )
{
  game_players_num=n;
}



/*
  Get level diamonds number to get
*/
int game_get_level_diamonds( void )
{
  return game_diamonds;
}
