/****************************************************************************

    sdl.h
    SDL interface

    DoubleDash game
      
    Copyright (C) 2003  Michal Benes
  
    DoubleDash is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.
  
    This code is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
  
    You should have received a copy of the GNU Lesser General Public
    License along with this code; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
                                                                               
*****************************************************************************/

#ifndef __SDL_H
#define __SDL_H


#include <SDL.h>
#include <SDL_keysym.h>

#define KEY_ENTER SDLK_RETURN
#define KEY_UP    SDLK_UP
#define KEY_DOWN  SDLK_DOWN
#define KEY_RIGHT SDLK_RIGHT
#define KEY_LEFT  SDLK_LEFT
#define KEY_Y     SDLK_Y




typedef struct {
  int images_num;
  SDL_Surface **image;  
} Sdl_Sprite;



/*
  Initialize SDL
 */
void sdl_init(void);




/*
  Quit SDL
 */
void sdl_close( void );



/*
  polls all pending events
*/
void sdl_poll_events( void );



/*
  resets keyboard status
*/
void sdl_reset_keyboard( void );


/* 
   wait for keypress
*/
int sdl_get_key( void );


/*
  return esc key state
 */
int sdl_esc_pressed( void );


/*
  return space key state
 */
int sdl_space_pressed( void );


/*
  load sprite image
*/
Sdl_Sprite *sdl_sprite_load( char *name, int columns, int rows );

/*
  free sprite image
*/
void sdl_sprite_free( Sdl_Sprite *sprite );


/*
  draw sprite
 */
void sdl_sprite_draw( Sdl_Sprite *sprite, int x, int y, int frame );


/*
  flip double buffer
 */
void sdl_flip( void );


/*
  set clip rectangle
 */ 
void sdl_clip( int x, int y, int w, int h );

/*
  resetset clip rectangle
 */ 
void sdl_unclip( void );


#endif /* __SDL_H */
