/****************************************************************************

    utils.c
    utility functions

    DoubleDash game
      
    Copyright (C) 2003  Michal Benes
  
    DoubleDash is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.
  
    This code is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
  
    You should have received a copy of the GNU Lesser General Public
    License along with this code; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
                                                                               
*****************************************************************************/


#include "doubledash.h"
#include "utils.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>


char utils_fullname_buf[1024];

/*
  allocates memory, exits on error
*/
void *utils_malloc( int size )
{
  void *result;
  result=malloc( size );
 
  if( result==NULL )
    {
      fprintf( stderr, "DoubleDash: Can not allocate memory %d bytes of memory\n", size );
      exit(1);
    }
   
  return result;
 
}


/*
  return full filename including datadir
 */
char *utils_fullname( char *name )
{
  sprintf( utils_fullname_buf, "%s/%s", DATADIR, name );
  return utils_fullname_buf;
}



/*
  trims whitespace characters from a buffer
*/
char *utils_trim( char *buf )
{
  char *result;
  char *end, *act;
  result=buf;
  while( *result && isspace(*result) ) ++result;
  act=end=result;
  while( *act )
    {
      /* eat nonspaces */
      while( *act && !isspace(*act) ) ++act;
      end=act;
      /* eat spaces */
      while( *act && isspace(*act) ) ++act;
    }
  *end=0;
  return result;
}


/*
  reads parameter value
*/
int utils_get_param( char *buf, char *name, int *value )
{
  char *b;
  if( strncmp( buf, name, strlen(name) ) ) return 0;

  b=buf+strlen(name);
  
  if( !isspace(b[0]) && !isdigit(b[0]) && b[0]!='=' ) return 0;

  *value=0;
  while( *b && !isdigit(*b) ) ++b;

  while( isdigit(*b) )
    {
      *value*=10;
      *value+=*b-'0';
      ++b;
    }

  return 1;
  
}



/*
  random number
*/
 
int utils_rnd( int n )
{
  int k;
  k=((double)n)*rand()/(RAND_MAX+1.0);
  if( k>=n ) k=n-1;
  if( k<0 ) k=0;
  return k;
}
